package cn.ischoolcode.javabook.scene9;

/* 
 * 爱校码
 * http://www.ischoolcode.cn/
 *  
 *  此代码的执行条件：
 *   1.  javac *.java
 *   2.  rmic ChatImpl
 *   3.  rmiregistry
 *   4.  打开一个新的命令提示符执行：  java ChatServer 端口号   
 *       应该响应：服务器运行的端口  [端口号] ....
 *   5.  从URL地址下载客户端代码：http://www.ischoolcode.cn/static/javabook/scene9/CRMIClient.java
 *   6.  打开一个新的命令提示符执行：javac RMIClient.java
 *   7.  运行一个新的客户端： java RMIClient 127.0.0.1 端口号 
 *   
 * @author zcj
 */
import java.rmi.*;
import java.rmi.server.*;
import java.net.*;
import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;

public class HMChatServer {
	private static HashMap<String, Socket> connectedUser = new HashMap<String, Socket>();
	private static Socket ClientSocket = null;
	private static ServerSocket serverSocket;
	private static String username = null;
	private static PrintWriter output;
	private static final String PUBLICMESSAGE = "PUBLICMESSAGE";
	private static final String ONLINE = "ONLINE";
	private static final String OFFLINE = "OFFLINE";

	public HMChatServer() {
		try {
			ChatImpl csi = new ChatImpl(this);
			Naming.rebind("rmi://localhost:1099/ChatService", csi);
		} catch (java.rmi.ConnectException ce) {
			JOptionPane.showMessageDialog(null,
					"问题 : 请运行RMI注册.", "连接异常",
					JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null,
					"问题 : 请运行RMI注册.", "异常",
					JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null,
					"问题 : 请运行RMI注册.", "异常",
					JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}
	}

	public static void main(String[] args) {
		try {
			int port = Integer.parseInt(args[0]);
			HMChatServer cs = new HMChatServer();
			cs.processConnection(port);
		} catch (ArrayIndexOutOfBoundsException ae) {
			JOptionPane.showMessageDialog(null, "请提供端口",
					"注意", JOptionPane.INFORMATION_MESSAGE);
			System.exit(-1);
		}
	}

	private void processConnection(int port) {
		try {
			serverSocket = new ServerSocket(port);
			System.out.println("服务器运行的端口 " + port + " .... ");
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null,
					"无法监听端口 " + port, "错误",
					JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}
		try {
			while (true) {
				addClient(serverSocket.accept());
				String username = getUsername();
				sendPublicMessage(PUBLICMESSAGE, "SERVER", "[" + username
						+ "] 现在线");
			}
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null, "不能接受连接.",
					"错误", JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}
	}

	public void connect(String username) {
		this.username = username;
	}

	public String getUsername() {
		return username;
	}

	public ArrayList getClientList() {
		ArrayList myUser = new ArrayList();
		Iterator i = connectedUser.keySet().iterator();
		String user = null;
		while (i.hasNext()) {
			user = i.next().toString();
			myUser.add(user);
		}
		return myUser;
	}

	public void addClient(Socket clientSocket) throws RemoteException {
		connectedUser.put(getUsername(), clientSocket);
		sendPublicMessage(ONLINE, getUsername(), "CLIENT");
	}

	public void sendPublicMessage(String keyword, String username,
			String message) throws RemoteException {
		Iterator i = connectedUser.keySet().iterator();
		String user = null;
		while (i.hasNext()) {
			try {
				user = i.next().toString();
				ClientSocket = connectedUser.get(user);
				output = new PrintWriter(ClientSocket.getOutputStream(), true);
				output.println(keyword + "***" + username + "***" + message);
				output.flush();
			} catch (IOException ioe) {
				connectedUser.remove(user);
				sendPublicMessage(OFFLINE, user, user
						+ " 已离开对话");
			}
		}
	}

	public void disconnect(String username) throws RemoteException {
		connectedUser.remove(username);
		sendPublicMessage(OFFLINE, username, username
				+ " 已离开对话");
		sendPublicMessage(PUBLICMESSAGE, "SERVER", username
				+ " 已离开对话");
	}
}
